/* $Id: natpmpc.c,v 1.0 2008/12/02 06:12:48 NoSFeRaTU Exp $ */
/* libnatpmp
 * Copyright (c) 2007, Thomas BERNARD <miniupnp@free.fr>
 * Copyright (c) 2008, NoSFeRaTU <master.nosferatu@gmail.com>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "natpmp.h"

/* sample code for using libnatpmp */
int main(int argc, char * * argv)
{
/*	printf("%d",argc);
	return 1;
*/
	if (argc<4)
	{
		if(argc>1 && strcmp(argv[1],"help") != 0 && strcmp(argv[1],"--help") != 0)
			printf("Sorry, not enough parameters!\n");
		printf("Usage: natpmpc <gateway> <protocol> <localport> [lifetime] [remoteport]\n\n");
		printf("    gateway: \"auto\" for autodetection or ip address of gateway\n");
		printf("   protocol: \"tcp\" or \"udp\"\n");
		printf("  localport: request mapping to this local port\n");
		printf("   lifetime: request mapping for this time\n");
		printf(" remoteport: request mapping to this remote port\n\n");
		printf(" If remoteport not specified then remoteport equal localport\n");
		return 1;
	} else {
		natpmp_t natpmp;
		natpmpresp_t response;
		int port1,port2,ltime;
		int r;
		//int sav_errno;
		struct timeval timeout;
		fd_set fds;

		r = initnatpmp(&natpmp,argv);
		if(r<0) {
			printf("ERROR: initnatpmp() returned %d\n", r);
			return 1;
		} else {
			printf("Initialization successfully finished, starting mapping request...\n");
		}

		r = sendpublicaddressrequest(&natpmp);
		if(r<0) {
			printf("ERROR: sendpublicaddressrequest returned %d\n", r);
			return 1;
		}

		do {
			FD_ZERO(&fds);
			FD_SET(natpmp.s, &fds);
			getnatpmprequesttimeout(&natpmp, &timeout);
			select(FD_SETSIZE, &fds, NULL, NULL, &timeout);
			r = readnatpmpresponseorretry(&natpmp, &response);
			/*sav_errno = errno;*/
			/*if(r<0 && r!=NATPMP_TRYAGAIN)
				printf("errno=%d '%s'\n",
				       sav_errno, strerror(sav_errno));*/
		} while(r==NATPMP_TRYAGAIN);
		if(r<0) {
			printf("ERROR: readnatpmpresponseorretry returned %d\n", r);
			return 1;
		}

		/* TODO : check that response.type == 0 */
		printf("Received public IP address: %s\n", inet_ntoa(response.publicaddress.addr));
//		printf("epoch = %u\n", response.epoch);

		port1=(argc>3 && argv[3] ? atoi(argv[3]) : 21000); 
		ltime=(argc>4 && argv[4] ? atoi(argv[4]) : 3600); 
		port2=(argc>5 && argv[5] ? atoi(argv[5]) : port1); 
		r = sendnewportmappingrequest(&natpmp, (strcmp(argv[2],"udp") ? NATPMP_PROTOCOL_UDP : NATPMP_PROTOCOL_TCP), port1, port2, ltime);
		if(r < 0) {
			printf("ERROR: sendnewportmappingrequest returned %d\n", r);
			return 1;
		}

		do {
			FD_ZERO(&fds);
			FD_SET(natpmp.s, &fds);
			getnatpmprequesttimeout(&natpmp, &timeout);
			select(FD_SETSIZE, &fds, NULL, NULL, &timeout);
			r = readnatpmpresponseorretry(&natpmp, &response);
		} while(r==NATPMP_TRYAGAIN);
		if(r<0) {
			printf("ERROR: readnatpmpresponseorretry returned %d\n", r);
			return 1;
		}
	
		/* TODO : check response.type ! */
		printf("Public port %hu successfully mapped to localport %hu with lifetime %u\n",
		       response.newportmapping.mappedpublicport,
			   response.newportmapping.privateport,
			   response.newportmapping.lifetime);
		//printf("epoch = %u\n", response.epoch);

		r = closenatpmp(&natpmp);
		if(r<0) {
			printf("ERROR: closenatpmp() returned %d\n", r);
			return 1;
		}

		return 0;
	}
}

